"""Top-level package for Snapchat Downloader."""

__author__ = """Aakash Gajjar"""
__email__ = "skyqutip@gmail.com"
__version__ = "0.2.7"

from snapchat_dl.snapchat_dl import SnapchatDL

__all__ = ["SnapchatDL"]
